@@ -1,3 +1,4 @@
+import contextlib
 import glob
 import inspect
 import itertools
@@ -81,13 +82,23 @@ class inject:
     :class:`class_property` instance pointing to `injected`.
     """
 
-    def __init__(self, name, injected):
+    @classmethod
+    def context(cls, name, injected):
+        """Shortcut to inject a contextual dependency in the function call
+        scope.
+        """
+        return cls(name, injected, scope='function')
+
+    def __init__(self, name, injected, scope='process'):
         self.attname = name
         self.injected = injected
+        self.scope = scope
 
     def __call__(self, obj):
         if inspect.isclass(obj):
             d = self._decorate_class(obj)
+        elif inspect.iscoroutinefunction(obj):
+            d = self._decorate_coroutinefunction(obj)
         elif inspect.isfunction(obj):
             d = self._decorate_function(obj)
         else:
@@ -105,6 +116,32 @@ class inject:
             return obj(*args, **kwargs)
         return f
 
+    def _decorate_coroutinefunction(self, obj):
+        @functools.wraps(obj)
+        async def f(*args, **kwargs):
+            dep = require(self.injected)
+            async with _ensure_async_context(dep, self.scope) as ctx:
+                kwargs[self.attname] = ctx
+                return await obj(*args, **kwargs)
+        return f
+
+
+@contextlib.asynccontextmanager
+async def _ensure_async_context(dep, scope):
+    if scope != 'function':
+        yield dep
+    else:
+        is_context_manager = hasattr(dep, '__aenter__')
+        try:
+            yield (await dep.__aenter__()) if is_context_manager else dep
+        except Exception as e:
+            if is_context_manager:
+                await dep.__aexit__(type(e), e, None)
+            raise
+        else:
+            if is_context_manager:
+                await dep.__aexit__(None, None, None)
+
 
 def call(name, *args, **kwargs):
     """Invoke the dependency identified by `name` with
